import polars as pl
from pathlib import Path as Path
from zipfile import ZipFile as ZipFile
import gzip

_supported_types_ = [".parquet", ".csv", ".csv.gz", ".zip",]

def dfdict_to_zip(df, path):
    with ZipFile(path,'a') as f:
        for filename, data in df.items():
            filetype = Path(filename)
            if filetype not in _supported_types_:
                raise ValueError(f'Invalid file format: {filetype}')
            elif(filetype == '.parquet'):
                s = df.write_parquet()
            elif(filetype == '.csv'):
                s = df.write_csv()
            elif(filetype == '.csv.gz'):
                s = gzip.compress(bytes(f'\ufeff{df.write_csv()}', 'utf-8'))
            else:
                raise ValueError(f'Invalid file format: {filetype} **DEBUG: else**')
            f.writestr(filename, s)
            
def df_to_file(df, path):
    path = Path(path)
    filetype = ''.join(path.suffixes)
    if filetype not in _supported_types_:
        raise ValueError(f'Invalid file format: {filetype}')
    elif(type(df) is dict):
        if(filetype == '.zip'):
            dfdict_to_zip(df, path)
        else:
            raise ValueError(f'You can only transfrom zip file to zip file')
    elif(filetype == '.parquet'):
        df.write_parquet(path)
    elif(filetype == '.csv'):
        df.write_csv(path)
    elif(filetype == '.csv.gz'):
        with open(path, 'rb') as f:
            f.write(gzip.compress(bytes(f'\ufeff{df.write_csv()}', 'utf-8')))
    elif(filetype == '.zip'):
        raise ValueError(f'You can only transfrom zip file to zip file')
    else:
        raise ValueError(f'Invalid file format: {filetype} **DEBUG: else**')
    return df



import polars as pl
import pandas as pd
from zipfile import ZipFile as ZipFile
import requests
import os
from pathlib import Path as Path

_supported_types_ = [".parquet", ".csv", ".csv.gz", ".zip",]

def is_url(data):
    try:
        if requests.get(url=data).status_code==200:
            return True
        else:
            return False
    except requests.exceptions.InvalidSchema:
        return False
    except requests.exceptions.MissingSchema:
        return False
def url_to_df(path):
    with open(path.name, 'wb') as f:
        f.write(request.get('data').content)
    df = any_to_df(path.name)
    os.remove(path.name)
    return df

def zip_to_dfdict(path):
    d = dict()
    infolist = Zipfile(path).infolist()
    for f in infolist:
        d[f.filename] = file_to_df(f.filename, Zipfile(path).read(name=f)) 
    return d

def file_to_df(path, f=None):
    filetype = ''.join(path.suffixes)
    if f is not None:
        path = f
    if filetype not in _supported_types_:
        raise ValueError(f'Invalid file format: {filetype}')
    elif(filetype=='.parquet'):
        df = pl.read_parquet(path)
    elif(filetype=='.csv'):
        df = pl.read_csv(path)
    elif(filetype=='.csv.gz'):
        df = pl.read_csv(path, gzip=True)
    elif(filetype=='.zip'):
        df = zip_to_dfdict(path)
    else:
        raise ValueError(f'Invalid file format: {filetype} **DEBUG: else**')
    return df

def any_to_df(data):
    if type(data) is pl.DataFrame():
        df = data
    elif type(data) is pd.DataFrame():
        df = pl.from_pandas(data)
    elif type(data) is str:
        path = Path(data)
        if is_url(data):
            df = url_to_df(path)
        else:
            df = file_to_df(path)
    else:
        raise ValueError(f'Invalid Input: {data}')
    return df


from any_to_df import any_to_df
from df_to_file import df_to_file
from df_operations import df_pivot, df_pick

def pivot(data, columns:str, values:str, filepath:str=None):
    df = any_to_df(data)
    if type(df) == dict:
        for _, f in df.items():
            f = df_pivot(f, columns, values)
    else:
        df = df_pivot(df, columns, values)
    if filepath is None:
        return df
    else:
        return df_to_file(df, filepath)

def melt(data, value_vars:str, variable_name:str=None, value_name:str=None, filepath:str=None):
    value_vars = value_vars.split(',')
    if variable_name == '':
        variable_name = None
    if value_name == '':
        value_name = None
    df=any_to_df(data)
    if type(df) == dict:
        for _, f in df.items():
            id_vars = [col for col in f.columns if col not in value_vars]
            f = f.melt(id_vars=id_vars, value_vars=value_vars, variable_name=variable_name, value_name=value_name)
    else:
        id_vars = [col for col in df.columns if col not in value_vars]
        df = df.melt(id_vars=id_vars, value_vars=value_vars, variable_name=variable_name, value_name=value_name)
    if filepath is None:
        return df
    else:
        return df_to_file(df, filepath)

def pick(data, col, value, filepath):
    df = any_to_df(data)
    if type(df) == dict:
        for _, f in df.items():
            f = df_pick(f, col, value)
    else:
        df = df_pick(df, col, value)
    if filepath is None:
        return df
    else:
        return df_to_file(df, filepath)


import cameo_table

def main():
    n = input('選擇想要的功能 輸入1:挑選出指定欄位為某值的所有列 輸入2:將直欄轉換為橫向資料 輸入3:將橫向資料轉換為直欄資料 輸入0:離開\n')
    while(n!='0'):
        if(n=='1'):
            print('已選功能1:挑選出指定欄位為某值的所有列')
            path, col_to_apply_on, value, result_path = input('以空白隔開 請依序輸入:要處理的檔名(路徑/url) 指定值所在的欄位 指定的值 輸出的檔名(路徑)\n').split(' ')
            try:
                cameo_table.pick(path, col_to_apply_on, value, result_path)
            except Exception as e:
                print(e)
        elif(n=='2'):
            print('已選功能2:將直欄轉換為橫向資料')
            path, col_to_apply_on, value, result_path = input('以空白隔開 請依序輸入:要處理的檔名(路徑/url) 轉換為欄位標題的欄位 對應值所在的欄位 輸出的檔名(路徑)\n').split(' ')
            try:
                cameo_table.pivot(path, col_to_apply_on, value, result_path)
            except Exception as e:
                print(e)
        elif(n=='3'):
            print('已選功能3:將橫向資料轉為直欄資料')
            data, value_vars, variable_name, value_name, filepath = input('以空白隔開 請依序輸入:要處理的檔名(路徑/url) 要合併為直欄的欄位(以逗號隔開) 合併後直欄的欄位標題 合併後值的欄位標題 輸出的檔名(路徑)\n').split(' ')
            try:
                cameo_table.melt(data=data, value_vars=value_vars, variable_name=variable_name, value_name=value_name, filepath=filepath)
            except Exception as e:
                print(e)
        n = input('選擇想要的功能 輸入1:挑選出指定欄位為某值的所有列 輸入2:將直欄轉換為橫向資料 輸入0:離開\n')

if __name__ == '__main__':
    main()


def df_pivot(df, columns, values):
    indexes = [col for col in df.columns if col not in values and col not in columns]
    counts = [df[index].n_unique() for index in indexes]
    ldf = df[indexes].distinct(subset=indexes).drop(indexes[counts.index(max(counts))])
    return ldf.hstack(df.pivot(values=values, index=indexes[counts.index(max(counts))], columns=columns))

def df_pick(df:'pl.DataFrame', column, value):
    return df[df[column]==value]